#ifndef __MATCHER_HEADER_INCLUDED__
#define __MATCHER_HEADER_INCLUDED__

#include <g_globals.h>
#include <packet.h>
#include <lists.h>
#include <targets.h>

/* Takes a parsed token and creates a private struct  */
typedef void * (*proc_match_validate)(char *);

/* Cleans up said parsed token when rule is unloaded */
typedef void   (*proc_match_cleanup)(void *);

/* Match a packet agains private data  */
typedef int (*proc_match_match)(void *, p_packet);

/* A matcher */
typedef struct matcher {
	list_head_t		l;
	char			*name;
	proc_match_validate	validate;
	proc_match_cleanup	cleanup;
	proc_match_match	match;
}matcher_t, *p_matcher;

/* Rule element */
typedef struct relement {
	list_head_t		l;
	void			*priv; /* cleanup with match->cleanup() */
	p_matcher		match;
}relement_t, *p_relement;

/* A rule object: 
* All criterion in a rule must be matched 
* positively. When matched, the packet is sent
* to the relevent target
*/
typedef struct rule {
	list_head_t		l;

	/* List of criteria to match */
	p_relement		criteria;

	p_target		target;
	void			*t_priv; /* cleanup with target->cleanup */
}rule_t, *p_rule;

/* Function declares */
int matcher_init(void);
void matcher_cleanup(int code, void *priv);
p_matcher matcher_get(char *);
void matcher_begin(p_packet);

/* Language plugins use these */
#ifndef __PLUGIN__
int matcher_add_rule(p_rule);
int matcher_add_criteria_to_rule(p_rule, char *, char *);
int matcher_set_rule_target(p_rule r, char *target, char *val);
#endif

typedef int (*proc_add_rule)(p_rule);
typedef int (*proc_add_criteria)(p_rule, char *, char *);
typedef int (*proc_set_target)(p_rule, char *, char *);

/* Function that plugins call to add new 
* matchers */
typedef int (*proc_matcher_add)(char *, 
		proc_match_validate,
		proc_match_cleanup,
		proc_match_match);
typedef void (*proc_rule_cleanup)(p_rule);

#ifndef __PLUGIN__
int matcher_add (char *, proc_match_validate,
	       proc_match_cleanup,
       	       proc_match_match);
void matcher_rule_cleanup(p_rule r);
#endif


#define SAFE_INIT_RULE_PTR(r) \
	r->target = NULL; \
	r->t_priv = NULL; \
	r->criteria = NULL; \
	r->l.next = NULL; \
	r->l.prev = NULL;

#endif //__MATCHER_HEADER_INCLUDED__
