#ifndef __PLUGINS_HEADER_INCLUDED__
#define __PLUGINS_HEADER_INCLUDED__

#include <g_globals.h>
#include <lists.h>

/* These are the subsystems we
* are plugins for  */
#include <capdev.h>
#include <demux.h>
#include <matcher.h>
#include <targets.h>
#include <rules.h>
#include <templates.h>

#define PLUGIN_ERR_OK		0x00000000	/* Success */
#define PLUGIN_ERR_FAIL		0x00000001	/* Non-specific failure */
#define PLUGIN_ERR_PARAM	0x00000002	/* Bad parameter */
#define PLUGIN_ERR_NOMEM	0x00000003	/* Not enough mem */
#define PLUGIN_ERR_SIZE		0x00000004	/* Wrong size object */

#define _PLUGIN_INIT		firestorm_plugin_init
#define PLUGIN_INIT		"firestorm_plugin_init"

#define _PLUGIN_UNLOAD		firestorm_plugin_unload
#define PLUGIN_UNLOAD		"firestorm_plugin_unload"


/* functions exported to plugin */
typedef int(*proc_print_out)(char *, ...);
typedef void(*proc_print_raw)(char *);

/* Struct we export to the plugin */
typedef struct plugin_in {
	/* MUST ALWAYS be first param */
	int			size;
	
	/* Plugins might want these to print
	* diagnostic information */
	proc_print_out		print_out;
	proc_print_raw		print_raw;

	/* Add capture devices */
	proc_add_captype	add_captype;

	/* Add a demux handler */
	proc_demux_add_handler	add_demux;

	/* Rule matchers */
	proc_matcher_add	add_matcher;

	/* Rule targets */
	proc_target_add		add_target;

	/* For rulesets */
	proc_add_rule		add_rule;
	proc_add_criteria	add_criteria;
	proc_set_target		set_target;
	proc_rule_cleanup	rule_cleanup;

	/* This is to add ruleset parsers */
	proc_add_parser		add_parser;

	/* Our linked list functions  */
	proc_list_append	list_append;
	proc_list_remove	list_remove;
	proc_list_prepend	list_prepend;
	
	/* Matcher/target/ruleset helper functions  */
	tmpl_t			templates;
}plugin_in_t, *p_plugin_in;

/* Data we import from the plugin */
typedef struct plugin_out {
	char		*name;		/* Plugin name */
	char		*desc;		/* Long description */
	uint		ver_major;	/* Version */
	uint		ver_minor;
}plugin_out_t, *p_plugin_out;

typedef uint(*proc_plugin_init)(p_plugin_in, p_plugin_out);
typedef uint(*proc_plugin_unload)(void);

/* Helper macros that all plugin init functions should call */
#define PLUGIN_CHECK_PARAM(x) \
	if ( x == NULL ) return PLUGIN_ERR_PARAM;

#define PLUGIN_CHECK_PARAMS(x) \
	PLUGIN_CHECK_PARAM(x) \
	if (x->size != sizeof(plugin_in_t)) return PLUGIN_ERR_SIZE; \
	PLUGIN_CHECK_PARAM(x->print_out) \
	PLUGIN_CHECK_PARAM(x->print_raw) \
	PLUGIN_CHECK_PARAM(x->add_captype) \
	PLUGIN_CHECK_PARAM(x->add_demux) \
	PLUGIN_CHECK_PARAM(x->add_matcher) \
	PLUGIN_CHECK_PARAM(x->add_target) \
	PLUGIN_CHECK_PARAM(x->add_rule) \
	PLUGIN_CHECK_PARAM(x->add_criteria) \
	PLUGIN_CHECK_PARAM(x->set_target) \
	PLUGIN_CHECK_PARAM(x->rule_cleanup) \
	PLUGIN_CHECK_PARAM(x->add_parser) \
	PLUGIN_CHECK_PARAM(x->list_append) \
	PLUGIN_CHECK_PARAM(x->list_prepend) \
	PLUGIN_CHECK_PARAM(x->list_remove)
	
#endif //__SENSOR_PLUGINS_HEADER_INCLUDED__
